#requires -version 2.0

 <#
 -----------------------------------------------------------------------------
 Script: Remove-File.ps1
 Version: 2.0
 Author: Jeffery Hicks
    http://jdhitsolutions.com/blog
    http://twitter.com/JeffHicks
    http://www.ScriptingGeek.com
 Date: 7/18/2011
 Keywords:
 Comments: You need to dot source this script to load the function
 into your PowerShell session.

 "Those who forget to script are doomed to repeat their work."

  ****************************************************************
  * DO NOT USE IN A PRODUCTION ENVIRONMENT UNTIL YOU HAVE TESTED *
  * THOROUGHLY IN A LAB ENVIRONMENT. USE AT YOUR OWN RISK.  IF   *
  * YOU DO NOT UNDERSTAND WHAT THIS SCRIPT DOES OR HOW IT WORKS, *
  * DO NOT USE IT OUTSIDE OF A SECURE, TEST SETTING.             *
  ****************************************************************
 -----------------------------------------------------------------------------
#>

Function Remove-File {

<#
.SYNOPSIS
Delete files and folders based on file age.
.DESCRIPTION
This function will go start at a specified path and delete files and folders that are older than a 
specified date and time. The comparison is made to the file's last modified time. The last access time
doesn't always accurately reflect when a file was last used. Typically you will use this function to
clean out temp folders.
.PARAMETER Path
The folder path to search. The default is the temp folder, $env:temp.
.PARAMETER Cutoff
The date time threshold. The default is the time the computer last booted.
.PARAMETER Recurse
Recursively search from the startting path.
.PARAMETER Hidden
Include hidden and system files files.
.PARAMETER Force
Force deletions without any prompting.

.EXAMPLE
PS C:\> Remove-File -recurse -force
Remove all files in the temp folder that are older than the computer's startup time.
.EXAMPLE
PS C:\> "C:\work","$env:temp","$env:windir\temp" | Remove-File -cutoff (Get-Date).AddDays(-30) -recurse -hidden -force 
This command will search the 3 specified paths for all files that haven't been modified in 30 days and delete them.
.NOTES
NAME        :  Remove-File
VERSION     :  2.0   
LAST UPDATED:  7/20/2011
AUTHOR      :  Jeffery Hicks
.LINK
http://jdhitsolutions.com/blog
.LINK
Get-ChildItem 
Remove-Item 
.INPUTS
String
.OUTPUTS
None
#>

[cmdletbinding(SupportsShouldProcess=$True)]

Param (
[Parameter(Position=0,ValueFromPipeline=$True,ValueFromPipelineByPropertyName=$True)]
[ValidateNotNullOrEmpty()]
[ValidateScript({Test-Path $_})]
[string[]]$Path=$env:temp,
[Parameter(Position=1)]
[datetime]$Cutoff=([System.Management.ManagementDateTimeConverter]::ToDateTime(([wmi]"root\cimv2:win32_OperatingSystem=@").LastBootUpTime)),
[Switch]$Recurse,
[Switch]$Hidden,
[Switch]$Force
)

Begin {    
    Write-Verbose "Removing files older than $cutoff"
    if ($Force) {
        Write-Verbose "...Force"
    }
    #create an array to store file information
    $files=@()
}

Process { 

 ForEach ($ItemPath in $Path) {   
    Write-Verbose "Processing $ItemPath"
    #define a basic command    
    $cmd="Get-ChildItem -path $ItemPath"
    
    #build on the command depending on what parameters where specified
    if ($recurse) {
        Write-Verbose "...Recurse"
        $cmd+=" -recurse"
    }
    
    if ($hidden) {
       Write-Verbose "...Hidden"
        $cmd+=" -force"
    }  
    
    # execute the command string filtering out directories
    Invoke-Expression -Command $cmd  | 
    where {-not $_.PSIsContainer -and $_.lastwritetime -lt $cutoff} | 
     foreach {
        #add current file to array
        $files+=$_
        
        if ($force) {
           Remove-Item -Path $_.fullname -force 
       }
        else {
           Remove-Item -Path $_.fullname 
       }     
    } #end forEach
  } #foreach $itempath  
 } #process

End {   
    #prepare a summary message
    if ($files) {
        $stats=$files | Measure-Object -Sum -Property length
        $msg="Attempted to delete {0} files for a total of {1:N4} MB ({2} bytes)" -f $stats.count,($stats.sum/1MB),$stats.sum
    }
    else {
        $msg="No matching files found."
    }
    
    Write-Host $msg -ForegroundColor Yellow
    Write-Verbose "Finished Cleanup"
}

} #end function



